use Module::Build;

use strict;

my $build = Module::Build->new(
    module_name => 'DBM::Deep',
    license => 'perl',
    requires => {
        'perl'              => '5.006_000',
        'Clone'             => '0.01',
        'Digest::MD5'       => '1.00',
        'Fcntl'             => '0.01',
        'FileHandle::Fmode' => '0.05',
        'Scalar::Util'      => '1.14',
    },
    optional => {
    },
    build_requires => {
        'File::Path'      => '0.01',
        'File::Temp'      => '0.01',
        'Test::Deep'      => '0.095',
        'Test::Warn'      => '0.08',
        'Test::More'      => '0.47',
        'Test::Exception' => '0.21',
        'IO::Scalar'      => '0.01',
    },
    create_makefile_pl => 'traditional',
    add_to_cleanup => [
        'META.yml', '*.bak', '*.gz', 'Makefile.PL', 't/test*.db', 'cover_db',
    ],
    test_files => 't/??_*.t',
);

$build->create_build_script;
